==> qsa-x11-free-1.0.0.tar.gz: qsa-x11-free-1.0.0/src/engine/qsglobal_object.cpp <==
/****************************************************************************
** $Id: qsglobal_object.cpp  1.0.0   edited Jun 20 21:19 $
**
** Copyright (C) 2001-2003 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid Qt Script for Applications license may use
** this file in accordance with the Qt Script for Applications License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsengine.h"
#include "qsobject.h"
#include "qsenv.h"
#include "qsoperations.h"
#include "qsinternal.h"
#include "qslexer.h"
#include "qsnodes.h"
#include "qsclass.h"
#include "qscheck.h"

#include "qsobject_object.h"
#include "qsfuncref.h"
#include "qsarray_object.h"
#include "qsbool_object.h"
#include "qsstring_object.h"
#include "qsnumber_object.h"
#include "qsmath_object.h"
#include "qsdate_object.h"
#include "qsregexp_object.h"
#include "qserror_object.h"

#include <stdio.h>
#include <string.h>

extern int qsyyparse();

using namespace QS;

static void postfixAdd( const QSClass *cl, QSCheckData *data )
{
    if( cl->enclosingClass() )
	postfixAdd( cl->enclosingClass(), data );
    // ### We should be doing correct enterXXX, but since we are not doing
    // any validation of the code, only add it for lookup by the eval
    // it should not make a difference.
    data->enterClass( (QSClass*) cl );
}

// eval()
static QSObject qsEval( QSEnv *env )
{
    QSObject x = env->arg( 0 );
    if ( !x.isString() ) {
	return x;
    } else {
	QSEngineImp *eimp = env->engine()->imp();
	QString s = x.toString();
	QSLexer::lexer()->setCode( s,
#ifdef QSDEBUGGER
				   eimp->sourceId()
#else
				   0
#endif
	    );
	QSNode::setFirstNode( eimp->firstNode());
	int yp = qsyyparse();
	eimp->setFirstNode( QSNode::firstNode() );
	eimp->setProgNode( QSProgramNode::last() );
	if ( yp || QSLexer::lexer()->lexerState() == QSLexer::Bad ) {
	    // ### stop this from growing
	    // ### (will be deleted at end of global eval)
	    return env->throwError( SyntaxError );
	}

	QSCheckData data( env );
	QSObject scopeTop = env->currentScope();
	postfixAdd( scopeTop.objectType(), &data );
	QSEvalScopeClass *cl = new QSEvalScopeClass( env->objectClass() );
	data.enterEval( cl );

	eimp->progNode()->check( &data );
	if ( data.hasError() )
	    return env->throwError( EvalError,
				    data.errorMessages().first(),
				    data.errorLines().first() );

	env->pushScope( cl->construct( QSList() ) );
	// ### delete tmp scope class

	QSObject res = eimp->progNode()->execute( env );

	env->popScope();

	if ( env->isReturnValueMode() )
	  return res;
	else if ( env->isNormalMode() ) {
	    if ( res.isValid() )
		return res;
	    else
		return env->createUndefined();
	} else
	    return res;
    }
}

// parseInt()
static QSObject qsParseInt( QSEnv *env )
{
    if( env->numArgs() < 1 )
	return QSObject();
    QString str = env->arg( 0 ).toString();
    int radix = env->arg( 1 ).toInt32();
    if ( radix == 0 )
	radix = 10;
    else if (radix < 2 || radix > 36 )
	return env->createNumber( NaN );
    return env->createNumber( str.toInt( 0, radix ) );
}

// parseFloat()
static QSObject qsParseFloat( QSEnv *env )
{
    QString str = env->arg( 0 ).toString();
    return env->createNumber( str.toDouble() );
}

// isNaN()
static QSObject qsIsNaN( QSEnv * env )
{
    return env->createBoolean( isNaN( env->arg( 0 ).toNumber() ) );
}

// isFinite()
static QSObject qsIsFinite( QSEnv *env )
{
    double n = env->arg( 0 ).toNumber();
    return env->createBoolean( !isNaN( n ) && !isInf( n ) );
}

// escape()
static QSObject qsEscape( QSEnv *env )
{
    const char non_escape[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	"abcdefghijklmnopqrstuvwxyz"
	"0123456789@*_+-./";
    QString r = "", str = env->arg( 0 ).toString();
    const QChar *c = str.unicode();
    for ( uint k = 0; k < str.length(); k++, c++ ) {
	int u = c->unicode();
	if ( u > 255 ) {
	    char tmp[7];
	    sprintf(tmp, "%%u%04X", u);
	    r += QString(tmp);
	} else if (strchr(non_escape, (char)u)) {
	    r += *c;
	} else {
	    char tmp[4];
	    sprintf(tmp, "%%%02X", u);
	    r += QString(tmp);
	}
    }
    return env->createString( r );
}

// unescape()
static QSObject qsUnescape( QSEnv *env )
{
    QString s, str = env->arg( 0 ).toString();
    int k = 0, len = str.length();
    while ( k < len ) {
	const QChar *c = str.unicode() + k;
	QChar u;
	if ( *c == QChar('%') && k <= len - 6 && *(c+1) == QChar('u') ) {
	    u = QSLexer::convertUnicode( (c+2)->unicode(), (c+3)->unicode(),
					 (c+4)->unicode(), (c+5)->unicode() );
	    c = &u;
	    k += 5;
	} else if ( *c == QChar('%') && k <= len - 3 ) {
	    u = QChar( QSLexer::convertHex( (c+1)->unicode(),
					    (c+2)->unicode() ) );
	    c = &u;
	    k += 2;
	}
	k++;
	s += QString(c, 1);
    }
    return env->createString( s );
}

QSGlobalClass::QSGlobalClass( QSClass *b )
  : QSWritableClass( b )
{
    // ### DontEnum
    addMember( "eval", QSMember( &qsEval ) );
    addMember( "parseInt", QSMember( &qsParseInt ) );
    addMember( "parseFloat", QSMember( &qsParseFloat ) );
    addMember( "isNaN", QSMember( &qsIsNaN ) );
    addMember( "isFinite", QSMember( &qsIsFinite ) );
    addMember( "escape", QSMember( &qsEscape ) );
    addMember( "unescape", QSMember( &qsUnescape ) );

    addStaticVariableMember( "NaN", createNumber(NaN), AttributePublic );
    addStaticVariableMember( "Infinity", createNumber(Inf), AttributePublic );
    addStaticVariableMember( "undefined", createUndefined(), AttributePublic );
}

Global::Global( QSEngine *e )
    : eng( e )
{
}

void Global::init()
{
    QSEnv *e = eng->rep->env();
    clss = e->globalClass();
    QSInstanceData *data = new QSInstanceData( clss->numVariables(),
					       e->createUndefined() );
    e->registerShared( data );
    setVal( data );

    eng->registerType( e->objectClass() );
    eng->registerType( e->funcRefClass() );
    eng->registerType( e->arrayClass() );
    eng->registerType( e->stringClass() );
    eng->registerType( e->booleanClass() );
    eng->registerType( e->numberClass() );
    eng->registerType( e->dateClass() );
    eng->registerType( e->regexpClass() );
    eng->registerType( e->typeClass() );
    eng->registerType( e->errorClass() );
    eng->registerType( e->debugClass() );
    eng->registerType( e->systemClass() );

    // other properties
    eng->registerType( e->mathClass() );

    e->pushScope( * this );
}

void Global::clear()
{
}

bool Global::putIntoNamespace( const QString &ns, const QString &p, const QSObject &o )
{
    QSObject obj = *this;
    QString s = ns;

    // break up dotted notation ns like P1.P2.P3
    while ( !s.isEmpty() ) {
	int pos = s.find( '.' );
	if ( pos < 0 )
	    pos = s.length();
	// don't overwrite any existing objects, reuse them
	QSObject o = obj.get( s.left( pos ) );
	if ( o.isDefined() ) {
	    obj = o;
	} else {
	    QSObject pkg = env()->objectClass()->construct();
	    obj.put( s.left( pos ), pkg /*, DontDelete*/ );
	    obj = pkg;
	}
	s = s.mid( pos + 1 );
    }

    obj.put( p, o );

    return TRUE;
}

QSObject Global::qualifiedGet( const QString &ns, const QString &p ) const
{
    QSObject nso = namespaceObject( ns );
    if ( nso.isDefined() )
	return nso.get( p );
    else
	return env()->createUndefined();
}

bool Global::hasQualifiedProperty( const QString &ns, const QString &p ) const
{
    return namespaceObject( ns ).hasProperty( p );
}

QSObject Global::namespaceObject( const QString &ns ) const
{
    QSObject obj = *this;
    QString s = ns;

    // break up dotted notation ns like P1.P2.P3
    while ( !s.isEmpty() ) {
	int pos = s.find( '.' );
	if ( pos < 0 )
	    pos = s.length();
	// don't overwrite any existing objects, reuse them
	obj = obj.get( s.left( pos ) );
	if ( !obj.isDefined() )
	    return env()->createUndefined();
	s = s.mid( pos + 1 );
    }

    return obj;
}

==> qsa-x11-free-1.1.0.tar.gz: qsa-x11-free-1.1.0/src/engine/qsglobal_object.cpp <==
/****************************************************************************
** $Id: qsglobal_object.cpp  1.1.0   edited 2004-01-27T23:32:40$
**
** Copyright (C) 2001-2003 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid Qt Script for Applications license may use
** this file in accordance with the Qt Script for Applications License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsengine.h"
#include "qsobject.h"
#include "qsenv.h"
#include "qsoperations.h"
#include "qsinternal.h"
#include "qslexer.h"
#include "qsnodes.h"
#include "qsclass.h"
#include "qscheck.h"

#include "qsobject_object.h"
#include "qsfuncref.h"
#include "qsarray_object.h"
#include "qsbool_object.h"
#include "qsstring_object.h"
#include "qsnumber_object.h"
#include "qsmath_object.h"
#include "qsdate_object.h"
#include "qsregexp_object.h"
#include "qserror_object.h"

#include <private/qmutexpool_p.h>

#include <stdio.h>
#include <string.h>

extern int qsyyparse();

using namespace QS;

static void postfixAdd( const QSClass *cl, QSCheckData *data )
{
    if( cl->enclosingClass() )
	postfixAdd( cl->enclosingClass(), data );
    // ### We should be doing correct enterXXX, but since we are not doing
    // any validation of the code, only add it for lookup by the eval
    // it should not make a difference.
    data->enterClass( (QSClass*) cl );
}

// eval()
static QSObject qsEval( QSEnv *env )
{
    QSObject x = env->arg( 0 );
    if ( !x.isString() ) {
	return x;
    } else {
	QSEngineImp *eimp = env->engine()->imp();
	QString s = x.toString();
	QSNode *progNode = 0;
	{ // begin mutext locker
#if defined (QT_THREAD_SUPPORT) && QT_VERSION >= 0x030300
        QMutexLocker locker(qt_global_mutexpool ? qt_global_mutexpool->get(QSLexer::lexer()) : 0);
#endif
	QSLexer::lexer()->setCode( s,
#ifdef QSDEBUGGER
				   eimp->sourceId()
#else
				   0
#endif
	    );
	int yp = qsyyparse();
	progNode = QSProgramNode::last();
	if ( yp || QSLexer::lexer()->lexerState() == QSLexer::Bad ) {
	    if (!progNode->deref())
		delete progNode;
	    return env->throwError( SyntaxError );
	}
	} // end mutext locker

	QSCheckData data( env );
	QSObject scopeTop = env->currentScope();
	postfixAdd( scopeTop.objectType(), &data );
	QSEvalScopeClass *cl = new QSEvalScopeClass( env->objectClass() );
	data.enterEval( cl );

	eimp->progNode()->check( &data );
	if ( data.hasError() ) {
	    if (!progNode->deref())
		delete progNode;
	    return env->throwError( EvalError,
				    data.errorMessages().first(),
				    data.errorLines().first() );
	}

	env->pushScope( cl->construct( QSList() ) );
	// ### delete tmp scope class

	QSObject res = eimp->progNode()->execute( env );

	if (!progNode->deref())
	    delete progNode;

	env->popScope();

	if ( env->isReturnValueMode() )
	  return res;
	else if ( env->isNormalMode() ) {
	    if ( res.isValid() )
		return res;
	    else
		return env->createUndefined();
	} else
	    return res;
    }
}

// parseInt()
static QSObject qsParseInt( QSEnv *env )
{
    if( env->numArgs() < 1 )
	return QSObject();
    QString str = env->arg( 0 ).toString();
    int radix;
    if (env->numArgs()==1) {
	if (str.startsWith("0x"))
	    radix = 16;
	else if (str.startsWith("0"))
	    radix = 8;
	else
	    radix = 10;
    } else {
	radix = env->arg( 1 ).toInt32();    
	if (radix < 2 || radix > 36 )
	    return env->throwError("parseInt failed; radix value must be between 2 and 36");
    }
    bool ok;
    int val = str.toInt(&ok, radix);
    if (!ok)
	return env->throwError(QString("parseInt failed; radix: %1, string: '%2'")
			       .arg(radix).arg(str));
    return env->createNumber(val);
}

// parseFloat()
static QSObject qsParseFloat( QSEnv *env )
{
    QString str = env->arg( 0 ).toString();
    return env->createNumber( str.toDouble() );
}

// isNaN()
static QSObject qsIsNaN( QSEnv * env )
{
    return env->createBoolean( isNaN( env->arg( 0 ).toNumber() ) );
}

// isFinite()
static QSObject qsIsFinite( QSEnv *env )
{
    double n = env->arg( 0 ).toNumber();
    return env->createBoolean( !isNaN( n ) && !isInf( n ) );
}

// escape()
static QSObject qsEscape( QSEnv *env )
{
    const char non_escape[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	"abcdefghijklmnopqrstuvwxyz"
	"0123456789@*_+-./";
    QString r = "", str = env->arg( 0 ).toString();
    const QChar *c = str.unicode();
    for ( uint k = 0; k < str.length(); k++, c++ ) {
	int u = c->unicode();
	if ( u > 255 ) {
	    char tmp[7];
	    sprintf(tmp, "%%u%04X", u);
	    r += QString(tmp);
	} else if (strchr(non_escape, (char)u)) {
	    r += *c;
	} else {
	    char tmp[4];
	    sprintf(tmp, "%%%02X", u);
	    r += QString(tmp);
	}
    }
    return env->createString( r );
}

// unescape()
static QSObject qsUnescape( QSEnv *env )
{
    QString s, str = env->arg( 0 ).toString();
    int k = 0, len = str.length();
    while ( k < len ) {
	const QChar *c = str.unicode() + k;
	QChar u;
	if ( *c == QChar('%') && k <= len - 6 && *(c+1) == QChar('u') ) {
	    u = QSLexer::convertUnicode( (c+2)->unicode(), (c+3)->unicode(),
					 (c+4)->unicode(), (c+5)->unicode() );
	    c = &u;
	    k += 5;
	} else if ( *c == QChar('%') && k <= len - 3 ) {
	    u = QChar( QSLexer::convertHex( (c+1)->unicode(),
					    (c+2)->unicode() ) );
	    c = &u;
	    k += 2;
	}
	k++;
	s += QString(c, 1);
    }
    return env->createString( s );
}

QSGlobalClass::QSGlobalClass( QSClass *b )
  : QSWritableClass( b )
{
    // ### DontEnum
    addMember( "eval", QSMember( &qsEval ) );
    addMember( "parseInt", QSMember( &qsParseInt ) );
    addMember( "parseFloat", QSMember( &qsParseFloat ) );
    addMember( "isNaN", QSMember( &qsIsNaN ) );
    addMember( "isFinite", QSMember( &qsIsFinite ) );
    addMember( "escape", QSMember( &qsEscape ) );
    addMember( "unescape", QSMember( &qsUnescape ) );

    addStaticVariableMember( "NaN", createNumber(NaN), AttributePublic );
    addStaticVariableMember( "Infinity", createNumber(Inf), AttributePublic );
    addStaticVariableMember( "undefined", createUndefined(), AttributePublic );
}

Global::Global( QSEngine *e )
    : eng( e )
{
}

void Global::init()
{
    QSEnv *e = eng->rep->env();
    clss = e->globalClass();
    QSInstanceData *data = new QSInstanceData( clss->numVariables(),
					       e->createUndefined() );
    e->registerShared( data );
    setVal( data );

    eng->registerType( e->objectClass() );
    eng->registerType( e->funcRefClass() );
    eng->registerType( e->arrayClass() );
    eng->registerType( e->stringClass() );
    eng->registerType( e->booleanClass() );
    eng->registerType( e->numberClass() );
    eng->registerType( e->dateClass() );
    eng->registerType( e->regexpClass() );
    eng->registerType( e->typeClass() );
    eng->registerType( e->errorClass() );
    eng->registerType( e->debugClass() );
    eng->registerType( e->systemClass() );

    // other properties
    eng->registerType( e->mathClass() );

    e->pushScope( * this );
}

void Global::clear()
{
}
